#
#
# $Id: rss.rb,v 1.3 2004/03/08 23:07:00 Yuukis Exp $
#
#
$KCODE = 'e'

=begin
ʬ Module 
=end

class RSSS
	MAX_LINE = 300
	NEXT = 'next'
	CSuffix = '.cache'

	def initialize(id, rss_array)
		@id = id
		@rsss = rss_array
		@local_fn = "#{CFG::SYS.root}#{@id}.rss#{CSuffix}"
		@local_next_fn = "#{CFG::SYS.root}rss.#{NEXT}/#{@id}.rss#{CSuffix}"

		flags = Struct.new('RSSFilterFlags', :update, :cache_plus)
		@flags = flags.new(false, false)

		Dir.mkdir("#{CFG::SYS.root}rss.#{NEXT}/") unless File.exist?("#{CFG::SYS.root}/rss.#{NEXT}/")
	end

	def get(update = false)
		update = true unless File.exist?(@local_fn)
		if update
			new_feeds = []
			@rsss.each {|rss| new_feeds << rss.get(update) }
			mkcache(new_feeds.flatten)
			finish
		end

		return(read)
	end

	def get_find(word)
		return(read(word))
	end

	def finish
		File.rename(@local_next_fn, @local_fn)
	end
	private :finish

	def rollback
		File.unlink(@local_next_fn)
	end

	def cached(uri)
		str = ''
		uregex = /#{Regexp.quote(uri)}/
		oldCache = IO.readlines(@local_fn).join
		nCache = open(@local_fn, 'w')
		nCache.flock(File::LOCK_EX)

		oldCache.each {|th|
			if th =~ uregex
				a = th.split('<>')
				th = a[0] + '<>C<>' + a[2] + '<>' +  a[3] + '<>' + a[4]
			end
			str << th
		}

		nCache.write str
		#nCache.flock(File::LOCK_UN)
		nCache.close
	end

	def mkcache(new_feeds)
		feeds = []
		old_links = []
		old_feeds = read
		old_feeds.each {|rss|
			rss['state']= ' '
			old_links << rss['link']
		}

		new_feeds.each {|rss|
			unless old_links.index(rss['link'])
				rss['state']= 'N'
				feeds << rss
			end
		}


		feeds << old_feeds
		feeds.flatten!
		feeds = feeds[0..MAX_LINE+1] if feeds.size > MAX_LINE + 1
		write(feeds)

		return(feeds)
	end
	private :mkcache

	def filter(array)  # not yet.
		x = array.clone
		if @flags.update
			y = /U|N/
			x.reject! {|i| i['state'] !~ y } 
		end
		x.reject! {|i| i['cache'] != '+' } if @flags.cache_plus
		x
	end
	private :filter

	def read(word = false)
		return([]) unless File.exist?(@local_fn)
		ret = []

		IO.readlines(@local_fn).each {|line|
			x = line.split('<>')
			if word
				if x[3] =~ /#{word}/
					ret << {'state' => x[0], 'cache' => x[1], \
						'site' => x[2], 'title' => x[3], 'link' => x[4]}
				end
			else
				ret << {'state' => x[0], 'cache' => x[1], \
					'site' => x[2], 'title' => x[3], 'link' => x[4]}
			end
		}

		return(filter(ret))
	end
	private :read

	def write(parsed_array)
		ret = ''
		parsed_array.each {|hash|
			['state', 'cache', 'site', 'title', 'link'].each {|key|
				ret << hash[key] + '<>'
			}
			ret.gsub!(/<>$/, '')
			ret.chomp!
			ret << "\n"
		}
		fh = open(@local_next_fn, 'w')
		fh.write ret
		fh.close
	end
	private :write
end


class TinyRSS
	Decoder = 'nkf -e'
	DSuffix = '.decoded'
	CSuffix = '.cache'
	DirSuffix = '.rss'
	RSSFileName = 'rss.dat'
	NEXT = 'next'

	attr_reader :site, :uri_top, :uri_rss, :dir

	def initialize(site, uri_top, uri_rss, dir)
		@site = site
		@uri_top = uri_top
		@uri_rss = uri_rss
		@dir = dir
		@local_fn = "#{CFG::SYS.root}#{@dir}/#{RSSFileName}"
		Dir.mkdir("#{CFG::SYS.root}#{@dir}/") unless File.exist?("#{CFG::SYS.root}#{@dir}/")
		Dir.mkdir("#{CFG::SYS.root}#{@dir}/#{NEXT}/") unless File.exist?("#{CFG::SYS.root}#{@dir}/#{NEXT}/")
	end

	def get(update = false)
		update = true unless File.exist?(@local_fn + CSuffix)
		if update
			past = read
			download
			return(cmp(past))
		else
			return(read)
		end
	end

	def download
		x, status = Net2ch.get_keep(@uri_rss, @local_fn)

		if $DEBUG
			Logger.write(__FILE__, __LINE__, "Getting RSS Feeds.")
			p @uri_rss
			p status
		end

		if x
			write(parse(@site, @local_fn))
		end
	end
	private :download

	def parse(site, fn)
		ret = []

		x = system("#{Decoder} #{fn} > #{fn}#{DSuffix}")
		if x
			decoded = IO.readlines("#{fn}#{DSuffix}").join
			decoded.sub!(/.*?<item /m, '<item ')
			decoded = decoded.split('</item>')
			decoded.pop
			decoded.each {|article|
				article_hash = {'site' => site , 'state' => ' ', 'cache' => ' ' }
				article.each {|line|
					case line
					when /^\s*<title>/ then article_hash['title']= line.gsub(/<.*?>/, '').chomp
					when /^\s*<link>/ then article_hash['link']= line.gsub(/<.*?>/, '').chomp
					end
				}
				article_hash['title'].gsub!(/^\s*/, '')
				article_hash['link'].gsub!(/^\s*/, '')
				ret << article_hash
			}
		end

		return(ret)
	end
	private :parse

	def cached(uri)
		str = ''
		uregex = /#{Regexp.quote(uri)}/
		oldCache = IO.readlines(@local_fn + CSuffix).join
		nCache = open(@local_fn + CSuffix, 'w')
		nCache.flock(File::LOCK_EX)

		oldCache.each {|th|
			if th =~ uregex
				a = th.split('<>')
				th = a[0] + '<>C<>' + a[2] + '<>' +  a[3] + '<>' + a[4]
			end
			str << th
		}

		nCache.write str
		#nCache.flock(File::LOCK_UN)
		nCache.close
	end

	def cmp(past_rss)
		past_link = []
		past_rss.each {|past| past_link << past['link'] }

		cur_rss = read
		cur_rss.reject! {|cur| past_link.index(cur['link']) }

		return(cur_rss)
	end
	private :cmp

	def read
		ret = []

		IO.readlines(@local_fn + CSuffix).each {|line|
			x = line.split('<>')
			ret << {'state' => x[0], 'cache' => x[1], \
				'site' => x[2], 'title' => x[3], 'link' => x[4]}
		}

		return(ret)
	end
	private :read

	def write(parsed_array)
		ret = ''
		parsed_array.each {|hash|
			['state', 'cache', 'site', 'title', 'link'].each {|key|
				ret << hash[key] + '<>'
			}
			ret.gsub!(/<>$/, '')
			ret.chomp!
			ret << "\n"
		}
		fh = open(@local_fn + CSuffix, 'w')
		fh.write ret
		fh.close
	end
	private :write
end
